requirejs(["assets/lib/require.min"], function() { //Add Require lib so that it gets compiled in
		
	/**
	 * Jquery-based plugin for creating toast-style messages
	 * GitHub: {@link https://github.com/joshuacainphone/JTC-Toast}
	 * Licenced under the MIT Licence
	 * @author Joshua T Cain
	 * @version 1.0.2.3
	 * @type {Object}
	 * @see https://github.com/joshuacainphone/JTC-Toast
	 */
	window.JtcToastClass = function () {
		const DEBUG = true;
		const STACKTRACE = false;
		const LOGGINGPREFIX = "JTC-Toast: \n"; //Prefix for all logged messages
		
		var toastMessageCounter = 0;
		var toastMessageHeightSum = 0;
		
		var eventTriggerSelectors = "toastr-trigger"; //Classes (comma separated) to attach trigger events to
		var eventCloserSelectors = "toastr-closer"; //Classes (comma separated) to add closer events to
		
		var elementClassSeen = "toastr-closed toastr-seen"; //Classes to be added to element(s) after closed/seen
		var elementClassContainer = "toastr-container"; //Class to be added to the container element
		var elementClassMessageWrapper = "toastr-message-wrapper"; //Class to be added to message wrapper elements
		var elementClassMessage = "toastr-message";
		var elementClassTimeoutWrapper = "toastr-timeout-wrapper";
		var elementClassTimeoutBar = "toastr-timeout-bar";
		var elementClassCloseButton = "toastr-close-button";
		
		var elementDataTarget = "toastr-target"; //Element data to look for, to use as target
		var elementDataTimeoutDuration = "toastr-timeout"; //Element data to look for, to use as timeout duration (seconds)
		var elementDataShowTimeoutBar = "toastr-showbar"; //Element data to look for, to decided whether to display timeoutbar or not
		
		var elementDataMessage = "toastr-message"; //Message for toast to display
		var elementDataMarkAsSeen = "toastr-seen"; //Whether to apply 'seen' class to element after it has been hidden
		
		var defaultTimeoutDuration = 5; //Wait before fading out
		var defaultOpacity = 0.98; //Opacity whilst displayed
		var defaultFadeOutSpeed = 300; //Milliseconds
		var defaultFadeInSpeed = 300; //Milliseconds
		var defaultShowTimeoutBar = false; //Whether to show timeout/loading bar
		var defaultMarkAsSeen = true; //Whether to apply 'seen' classes to element once hidden
		
		/**
		 * Sets up recursive JS Function Stack Trace
		 * @since 1.0.0
		 * @type {Function}
		 * @author Helen Emerson
		 * @return {Null}
		 * @see http://helephant.com/2007/05/12/diy-javascript-stack-trace
		 */
		var initStacktrace = function() {
			Function.prototype.trace = function () {
				var trace = [];
				var current = this;
				while (current) {
					trace.push(current.signature());
					current = current.caller;
				}
				return trace;
			};
			Function.prototype.signature = function () {
				var signature = {
					name: this.getName(),
					params: [],
					toString: function () {
						var params = this.params.length > 0 ? "'" + this.params.join("', '") + "'" : "";
						return this.name + "(" + params + ")"
					}
				};
				if (this.arguments) {
					for (var x = 0; x < this.arguments.length; x++) signature.params.push(this.arguments[x]);
				}
				return signature;
			};
			Function.prototype.getName = function () {
				if (this.name) return this.name;
				var definition = this.toString().split("\n")[0];
				var exp = /^function ([^\s(]+).+/;
				if (exp.test(definition)) return definition.split("\n")[0].replace(exp, "$1") || "anonymous";
				return "anonymous";
			};
		};
		initStacktrace();
		
		/**
		 * Log a message to the client console
		 * @since 1.0.0
		 * @type {Function}
		 * @param {String} message - Message to be logged
		 * @return {Null} null
		 * @since 1.0.0
		 */
		var logMessage = function(message) {
			message = String(message) || "";
			var datestamp = new Date();
			var loggingSuffix = "\n(" + String(datestamp) + ")";
			if (message != "" && window.console.log) window.console.log(LOGGINGPREFIX + message + loggingSuffix);
		};
		
		/**
		 * Log a warning message to the client console
		 * @since 1.0.0
		 * @type {Function}
		 * @param {String} message - Warning message to be logged
		 * @return {Null} null
		 */
		var logWarning = function(message) {
			message = String(message) || "";
			var datestamp = new Date();
			var loggingSuffix = "\n(" + String(datestamp) + ")";
			if (message != "" && window.console.warn) {
				window.console.warn(LOGGINGPREFIX + message + loggingSuffix);
			}
			else {
				logMessage("[Warning] " + message);
			}
		};
		
		/**
		 * Log an error message to the client console
		 * @since 1.0.0
		 * @type {Function}
		 * @param {String} message - Error message to be logged
		 * @return {Null} null
		 */
		var logError = function(message) {
			message = String(message) || "";
			var datestamp = new Date();
			var loggingSuffix = "\n(" + String(datestamp) + ")";
			if (message != "" && window.console.error) {
				window.console.error(LOGGINGPREFIX + message + loggingSuffix);
				logDebugMessage(message);
			}
			else {
				logMessage("[Error] " + message);
			}
		};
		
		/**
		 * Log a message to the client console to aid debugging
		 * @since 1.0.0
		 * Hidden if DEBUG is False
		 * Shows stacktrace if STACKTRACE is True
		 * @type {Function}
		 * @param {String} message - Message to be logged
		 * @param {Boolean} [stacktracing=false] - Whether to include stacktrace
		 * @return {Null} null
		 */
		var logDebugMessage = function(message, stacktracing) {
			message = String(message) || "";
			stacktracing = stacktracing || false;
			if(DEBUG) {
				if(STACKTRACE || stacktracing) {
					var stacktrace = arguments.callee.trace();
					var stacktraceString = stacktrace.join("\n\t");
					logMessage(message + " \nStacktrack:\n\t" + stacktraceString);
				}
				else {
					logMessage(message);
				}
			}
		};
		
		//JQuery check
		if(!(window.jQuery || window.$)){
			logError("window.jQuery is not defined.");
			return false;
		}
		
		/**
		 * Set up trigger classes for opening toast messages
		 * @since 1.0.1
		 * @type {Function}
		 * @return {Null}
		 */
		var setOpenerTriggers = function(){
			//Assign triggers for showing toast messages
			$("." + eventTriggerSelectors).on("click", function(){
				publicFunctionality.displayToast(
					$(this).data(elementDataMessage),
					$(this).data(elementDataTimeoutDuration),
					$(this).data(elementDataShowTimeoutBar)
				);
			});
		};
		setOpenerTriggers();
		
		/**
		 * Set up trigger classes for closing toast messages
		 * @since 1.0.1
		 * @type {Function}
		 * @return {Null}
		 */
		var setCloserTriggers = function(){
			//Assign triggers for hiding toast message
			$("." + eventCloserSelectors).on("click", function(){
				hideToastElement(
					$(this).data(elementDataTarget),
					$(this).data(elementDataMarkAsSeen)
				);
			});
		};
		
		/**
		 * Create toast message element
		 * @since 1.0.1
		 * @type {Function}
		 * @param {String} message - Message to be displayed in the toast
		 * @return {?String} elementId - Returns ID attribute of created element if successful, or Null.
		 */
		var createToastElement = function(message){
			message = String(message) || "";
			if(message != "") {
				
				if((toastMessageHeightSum+160) < $(window).height()) {
					
					var elementId = "toastr-" + toastMessageCounter;
					var dataTarget = "#" + elementId;
					
					//Create elements
					var containerElement = $(document.createElement('div'));
					var messageWrapperElement = $(document.createElement('div'));
					var timeoutWrapperElement = $(document.createElement('div'));
					var timeoutBarElement = $(document.createElement('div'));
					
					//Container
					containerElement.addClass(elementClassContainer + " toastr-hidden");
					containerElement.attr("id", elementId);
					containerElement.attr("data-" + elementDataTarget, dataTarget);
					containerElement.css({"bottom": toastMessageHeightSum + "px"}); //Check height of each message
					
					//Message
					messageWrapperElement.addClass(elementClassMessageWrapper);
					messageWrapperElement.attr("id", elementId + "-body");
					messageWrapperElement.attr("data-" + elementDataTarget, dataTarget);
					messageWrapperElement.append("<p class='" + elementClassMessage + "' data-" + elementDataTarget + "='" + dataTarget + "'>" + message + "</p>");
					//messageWrapperElement.append("<p class='" + elementClassMessage + "' data-" + elementDataTarget + "='" + dataTarget + "'>" + message + "</p>");
					messageWrapperElement.append("<p class='" + elementClassCloseButton +  " " + eventCloserSelectors + "' data-" + elementDataTarget + "='" + dataTarget + "'>X</p>");
					
					//Timeout bar
					timeoutWrapperElement.addClass(elementClassTimeoutWrapper);
					timeoutBarElement.addClass(elementClassTimeoutBar);
					
					//Put it all together
					messageWrapperElement.appendTo(containerElement);
					timeoutBarElement.appendTo(timeoutWrapperElement);
					timeoutWrapperElement.appendTo(containerElement);
					containerElement.appendTo("body");
					
					toastMessageHeightSum += $(dataTarget).height();
					toastMessageCounter++;
					setCloserTriggers(); //reapply triggers because this element is new.
					
					return elementId;
				}
				return null;
			}
			else {
				logError("Cannot create toast element without a valid message");
				return null;
			}
		};
		
		/**
		 * Display a toast message
		 * @since 1.0.1
		 * @type {Function}
		 * @param {String} target - CSS Selector (element ID) including '#' prefix
		 * @param {Number} [timeout=defaultTimeoutDuration] - (Seconds) Time to wait before hiding message
		 * @param {Boolean} [showTimeoutBar=defaultShowTimeoutBar] - (Seconds) Whether to show timeout/loading bar
		 * @return {?Boolean} success - Returns False if error occurred
		 */
		var showToastElement = function (target, timeout, showTimeoutBar) {
			target = String(target) || null;
			timeout = timeout || defaultTimeoutDuration;
			showTimeoutBar = showTimeoutBar || defaultShowTimeoutBar;
			
			//Check var type
			if(isNaN(timeout)){
				logWarning("showToastElement(): timeout not set to a number.\nDefault value used.");
				timeout = defaultTimeoutDuration;
			}
			
			if (target) {
				$(target).addClass("toastr-appearing");
				
				$(target).fadeTo(defaultFadeInSpeed, defaultOpacity).promise().done(function() {
					$(target).removeClass("toastr-appearing");
					$(target).removeClass("toastr-hidden");
					$(target).addClass("toastr-visible");
					if(timeout > 0){
						if(showTimeoutBar) startTimeoutBar(target, timeout);
						setTimeout(function(){
							hideToastElement(target);
						}, (timeout * 1000));
					}
					if(timeout <= 0 && showTimeoutBar){
						startTimeoutBar(target, timeout);
						setTimeout(function(){
							hideToastElement(target);
						}, (timeout * 1000));
					}
				});
				return true;
			}
			else {
				logWarning("showToastElement(): No target set.");
				return false;
			}
		};
		
		/**
		 * Show loading/timeout bar
		 * @since 1.0.0
		 * @param {String} target - CSS Selector (element ID) including '#' prefix
		 * @param {Number} [timeout=defaultTimeoutDuration] - (Seconds) Time to wait before hiding message
		 * @return {?Boolean} success - Returns False if error occurred
		 */
		var startTimeoutBar = function(target, timeout){
			target = String(target) || null;
			timeout = timeout || defaultTimeoutDuration;
			
			if(target && !isNaN(timeout)) {
				$("#" + $(target).attr('id') + " ." + elementClassTimeoutWrapper).show();
				$("#" + $(target).attr('id') + " ." + elementClassTimeoutBar).animate({
					width: "0px",
					marginLeft: "100%"
				}, (timeout * 1000));
			}
			else {
				return false;
			}
		};
		
		/**
		 * Reset loading/timeout bar styles
		 * @since 1.0.0
		 * @param {String} target - CSS Selector (element ID) including '#' prefix
		 * @return {?Boolean} success - Returns False if error occurred
		 */
		var resetTimeoutBar = function(target){
			target = String(target) || null;
			if(target) {
				$("#" + $(target).attr('id') + " ." + elementClassTimeoutWrapper).hide();
				$("#" + $(target).attr('id') + " ." + elementClassTimeoutBar).css({
					width: "100%",
					marginLeft: "0px"
				});
			}
			else {
				return false;
			}
		};
		
		/**
		 * Hide a toast message
		 * @since 1.0.0
		 * @version 1.0.2.3
		 * @type {Function}
		 * @param {String} target - CSS Selector (element ID) including '#' prefix
		 * @param {Boolean} [markAsSeen=defaultMarkAsSeen] - Whether to add the 'seen' class to the target after it has been hidden
		 * @return {?Boolean} success - Returns False if error occurred
		 */
		var hideToastElement = function (target, markAsSeen) {
			target = String(target) || null;
			markAsSeen = markAsSeen || defaultMarkAsSeen;
			
			//Validate input
			if(typeof markAsSeen != "boolean") {
				logWarning("hideToastElement(): markAsSeen now set as boolean value.\nDefault value used.");
				markAsSeen = defaultMarkAsSeen;
			}
			
			if (target) {
				
				$(target).addClass("notification-disappearing");
				$(target).fadeOut(defaultFadeOutSpeed).promise().done(function () {
					$(target).removeClass("toastr-visible");
					$(target).removeClass("notification-disappearing");
					$(target).addClass("toastr-hidden");
					
					if (markAsSeen) $(target).addClass(elementClassSeen);
					
					resetTimeoutBar(target);
					
					toastMessageHeightSum -= $(target).height();
					
					$("." + elementClassContainer).each(function(){
						var thisBottom = 0;
						var targetBottom = 0;
						
						try {
							thisBottom = $(this).css("bottom") || 0;
							thisBottom = String(thisBottom).replace("px", "");
							thisBottom = +thisBottom;
						} catch(ex) { logDebugMessage(String(ex)); }
						
						try {
							targetBottom = $(target).css("bottom") || 0;
							targetBottom = String(targetBottom).replace("px", "");
							targetBottom = +targetBottom + $(target).height();
						} catch(ex) { logDebugMessage(String(ex)); }
						
						if (thisBottom >= targetBottom && (thisBottom - $(target).height()) >= 0) {
							$(this).animate({"bottom": (thisBottom - $(target).height())});
						}
					}).promise().done(function(){
						$(target).remove();
						return true;
					});
				});
			}
			else {
				logWarning("hideToastElement(): No target set.");
				return false;
			}
		};
		
		//Public functionality
		var publicFunctionality = {
			
			/**
			 * Display toast message
			 * @since 1.0.1
			 * @type {Function}
			 * @param {String} message - The message to be displayed
			 * @param {Number} [timeout=null] - (Seconds) Time to wait before hiding message
			 * @param {Boolean} [showTimeOut=null] - Whether to show timeout/loading bar
			 * @return {Boolean} success - Returns false if an error occurred
			 */
			displayToast: function(message, timeout, showTimeOut){
				message = String(message) || "";
				timeout = timeout || defaultTimeoutDuration;
				showTimeOut = showTimeOut || defaultShowTimeoutBar;
				
				//Validate input
				if(isNaN(timeout)){
					logWarning("'timeout' not set to a Number value" +
						"\nToast Message: \"" + message + "\"" +
						"\nDefault value used.");
					timeout = defaultTimeoutDuration;
				}
				if(typeof showTimeOut != "boolean"){
					logWarning("showTimeOut not set to a Boolean value" +
						"\nToast Message: \"" + message + "\"" +
						"\nDefault value used.");
					showTimeOut = defaultShowTimeoutBar;
				}
				
				if(message != ""){
					showToastElement( "#" + createToastElement(message), timeout, showTimeOut );
					return true;
				}
				else {
					logWarning("displayToast(): No message provided.");
					return false;
				}
			},
			
			// var defaultFadeOutSpeed = 300; //Milliseconds
			// var defaultFadeInSpeed = 300; //Milliseconds
			// var defaultShowTimeoutBar = false; //Whether to show timeout/loading bar
			// var defaultMarkAsSeen = true; //Whether to apply 'seen' classes to element once hidden
			
			/**
			 * Get or set default fade-out speed
			 * @param {Number} [value=defaultFadeOutSpeed] - Value to set as the new defaultFadeOutSpeed
			 * @return {number} defaultFadeOutSpeed
			 */
			defaultFadeOutSpeed: function(value){
				var value = value || defaultFadeOutSpeed;
				if(!isNan(value)){
					if(value >= 0) defaultFadeOutSpeed = value;
				}
				return defaultFadeOutSpeed;
			},
			
			/**
			 * Get or Set toast box defaultOpacity
			 * @param {Number} [value=defaultOpacity] - New value to set as the defaultOpacity (must be between 0 and 1)
			 * @return {Number} defaultOpacity
			 */
			defaultOpacity: function(value){
				var value = value || defaultOpacity;
				if(!isNaN(value)){
					if(value >= 0 && value <= 1) defaultOpacity = value;
				}
				return defaultOpacity;
			},
			
			/**
			 * Get or Set default timeout duration for toasts
			 * @param {Number} [value=defaultTimeoutDuration] - New value to set as the defaultTimeoutDuration
			 * @return {Number}
			 */
			defaultTimeoutDuration: function(value){
				var value = value || defaultTimeoutDuration;
				if(!isNaN(value)) defaultTimeoutDuration = value;
				return defaultTimeoutDuration;
			},
			
			/**
			 * Get number of messages that have been displayed
			 * @return {number} toastMessageCounter - number of messages shown
			 */
			getCounter: function(){ return toastMessageCounter; },
			
			/**
			 * Get total height of all messages currently being displayed
			 * @return {number} toastMessageHeightSum - total height of the message stack
			 */
			getTotalHeightStack: function(){ return toastMessageHeightSum; },
			
			/**
			 * Find whether debugging is turned on
			 * @return {Boolean} DEBUG
			 */
			isDebug: function(){ return DEBUG; },
			
			/**
			 * Find whether stacktracing is turned on
			 * @return {boolean}
			 */
			hasStacktrace: function(){ return STACKTRACE; }
			
		};
		
		return publicFunctionality;
	};
		
	window.jtctoastr = new JtcToastClass();
	jtctoastr.displayToast("Messages have been typed", 15, true);
});
